/*
	imlabel.cpp
	
	QButton representing an IM contact
	
	Copyright (c) 2004 Will Stephenson   <lists@stevello.free-online.co.uk>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <qbitmap.h>
#include <qdragobject.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qpopupmenu.h>
#include <qtooltip.h>
//#include <qbuffer.h>
#include <kfiledialog.h>
#include <kiconeffect.h>
#include <kiconloader.h>
#include <kio/netaccess.h>
#include <kinputdialog.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kurldrag.h>

#include "kimproxy.h"

#include "imlabel.h"

IMLabel::IMLabel( QWidget *parent, KIMProxy *proxy ) : IMLabelBase( parent )
{
	m_proxy = proxy;
	connect( m_proxy, SIGNAL( sigContactStatusChanged( QString ) ), this, SLOT( statusChanged( const QString& ) ) );
	connect( this, SIGNAL( clicked() ), this, SLOT( chat() ) );
	//setPixmap( SmallIcon( "metacontact_unknown" ) );
	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	//setScaledContents( true );
	initMenu();
	setAcceptDrops( true );
}

IMLabel::IMLabel( QWidget *parent, KIMProxy *proxy, const KABC::Addressee& addressee )
	: IMLabelBase( parent )
{
	m_proxy = proxy;
	connect( m_proxy, SIGNAL( sigContactStatusChanged( QString ) ), this, SLOT( statusChanged( ) ) );
	connect( this, SIGNAL( clicked() ), this, SLOT( chat() ) );
	m_addressee = addressee;
	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	//setScaledContents( true );
	initMenu();
	setAcceptDrops( true );
}

IMLabel::~IMLabel()
{
}

void IMLabel::setAddressee( const KABC::Addressee& addressee )
{
	m_addressee = addressee;
	//KMessageBox::information(0, m_addressee.uid() );
	statusChanged( m_addressee.uid() );
}

KABC::Addressee IMLabel::addressee()
{
	return m_addressee;
}

void IMLabel::statusChanged( const QString& uid )
{
	// check the status change is for our contact
	if ( !m_addressee.isEmpty() && m_addressee.uid() == uid )
	{
		QToolTip::remove( this );
		// update our tooltip
		QString text = 
			QString("%1 : %2").arg( m_addressee.realName() ).arg( m_proxy->statusString( m_addressee.uid() ) );
		QToolTip::add( this, text );

		// update our picture
		QPixmap pic = getAddressBookPicture();
		if ( pic.isNull() )
		{
			pic = m_proxy->statusIcon ( m_addressee.uid() );
		}
		m_image->setPixmap( pic );
		m_text->setText( m_proxy->statusString( m_addressee.uid() ) );
	}
}

void IMLabel::chat()
{
	if ( !m_addressee.isEmpty() )
	{
		m_proxy->chatContactById( m_addressee.uid() );
	}
}

void IMLabel::message()
{
	if ( !m_addressee.isEmpty() )
	{
		QString message = KInputDialog::getText( "Enter message", "Message:" );
		m_proxy->messageContactById( m_addressee.uid(), message );
	}
}
// void IMLabel::resizeEvent( QResizeEvent *e )
// {
// 	QLabel::resizeEvent( e );
// 	QPixmap pic = getAddressBookPicture();
// 	if ( pic.isNull() && !m_addressee.isEmpty() )
// 	{
// 		pic = m_proxy->statusIcon ( m_addressee.uid() );
// 	}
// 	setPixmap( pic );
// }

void IMLabel::sendFile()
{
	if ( !m_addressee.isEmpty() )
	{
		QString file = KFileDialog::getOpenFileName( QString::null, QString::null, 0, i18n( "Choose file to send:" ) );
		KURL url;
		url.setPath( file );
		m_proxy->sendFileToId( m_addressee.uid(), url );
	}
}

QPixmap IMLabel::getAddressBookPicture()
{
	QPixmap pixmap;
	KABC::Picture photo = m_addressee.photo();
	if ( photo.isIntern() ) {
		pixmap = photo.data();
	} else {
		KURL url = photo.url();
		QString tempFile;
		if ( !url.isEmpty() )
		{
			if ( url.isLocalFile() )
				pixmap = QPixmap( url.path() );
			else if ( KIO::NetAccess::download( url, tempFile, this ) ) {
				pixmap = QPixmap( tempFile );
				KIO::NetAccess::removeTempFile( tempFile );
			}
		}
	}

	if ( !pixmap.isNull() )
	{
		QImage image = pixmap.convertToImage();
		// need to scale to available space only, not whole size of IMLabel
		image = image.smoothScale( m_image->size(), QImage::ScaleMin );
		pixmap.convertFromImage( image );

		// statusify?
		switch ( m_proxy->statusNumeric( m_addressee.uid() ) )
		{
			case -1:
			case 0:
				// Status unknown or unknown contact
				pixmap = SmallIcon( "metacontact_unknown" );
				break;
			case 1:
			case 2:
				// Offline or connecting
				pixmap = KIconEffect().apply( pixmap, KIconEffect::ToGray, 0.99, Qt::black, false);
				break;
			case 3:
				{	
					// Away
					QPainter b( &pixmap );
					b.setPen( QPen( Qt::white, 3 ) );
					//b.setBrush( Qt::white );
					//b.drawEllipse( pixmap.width() / 2, 0, pixmap.width() / 2, pixmap.height() / 2 );
					b.drawLine( pixmap.width(), 0, 0, pixmap.height() );
					break;
				}
			case 4:
				// Online
				break;
		}
	}
	return pixmap;
}

QPixmap IMLabel::renderIcon( QPixmap base, const QString& overlay, bool idle ) const
{
	// create an icon suiting the status from the base icon
	
	//composite the iconOverlay for this status and the supplied baseIcon
	if ( !overlay.isEmpty() ) // otherwise leave the base as-is
	{
		QPixmap overlayPic = SmallIcon( overlay );
		// first combine the masks of both pixmaps
/*		if ( overlayPic.mask() )
		{
			QBitmap mask = QBitmap( *(base.mask() ) );
			bitBlt( &mask, 0, 0, const_cast<QBitmap *>(overlayPic.mask()), 0, 0, overlayPic.width(), overlayPic.height(), Qt::OrROP );

			base.setMask( mask);
		}*/
		// draw the overlay on top of it
		QPainter p( &base );
		p.drawPixmap( 0, 0, overlayPic );
	}

	// if idle, apply effects
	if ( idle )
		KIconEffect::semiTransparent( base );

	return base;
}

void IMLabel::initMenu()
{
	m_menu = new QPopupMenu( this, "rmb_menu" );
	m_menu->insertItem( i18n( "Send &Message..." ), this, SLOT( message() ) );
	m_menu->insertItem( i18n( "Start &Chat..." ), this, SLOT( chat() ) );
	m_menu->insertItem( i18n( "Send &File..." ), this, SLOT( sendFile() ) );
}

void IMLabel::mousePressEvent( QMouseEvent *e )
{
	if ( e->button() == RightButton )
		m_menu->popup( e->globalPos() );
	else
		chat();
}

void IMLabel::dragEnterEvent( QDragEnterEvent* event )
{
	event->accept( KURLDrag::canDecode( event ) );
}

void IMLabel::dropEvent( QDropEvent* event )
{
	if ( !KURLDrag::canDecode( event ) )
		return;

	KURL::List uris;
	if ( KURLDrag::decode( event, uris ) && ( uris.count() > 0 ) ) {
		// TODO: Download remote file
		if (uris.first().isLocalFile())
			// What if the IMLabel and the source of the file are different
			// machines? X makes it possible.
			m_proxy->sendFileToId( m_addressee.uid(), uris.first() );
	}
}

#include "imlabel.moc"
